/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.databasebuinstaller;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.tinygroup.database.config.table.Table;
import org.tinygroup.database.table.TableProcessor;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.springutil.SpringUtil;
import org.tinygroup.vfs.FileObject;
import org.springframework.jdbc.core.JdbcTemplate;

public class TableInstaller extends DbInstaller {

	private Logger logger = LoggerFactory.getLogger(TableInstaller.class);
	private List<Table> tableList = new ArrayList<Table>();
	private TableProcessor tableProcessor;

	public TableProcessor getTableProcessor() {
		return tableProcessor;
	}

	public void setTableProcessor(TableProcessor tableProcessor) {
		this.tableProcessor = tableProcessor;
	}

	public void process(FileObject fileObject) {
		logger.logMessage(LogLevel.INFO, "开始install BU包中的表格,bu包:{0}",
				fileObject.getAbsolutePath());
		List<Table> list = tableProcessor.getTables();
		List<String> sqls = new ArrayList<String>();

		SessionFactory sessionfactory = (SessionFactory) SpringUtil
				.getBean("sessionFactory");
		logger.logMessage(LogLevel.INFO, "获得sessionFactory");
		Session session = sessionfactory.openSession();
		Connection connect = session.connection();
		logger.logMessage(LogLevel.INFO, "获得数据库connect");
		for (Table table : list) {
			deal(table, sqls, connect);
		}
		if (sqls.size() == 0) {
			logger.logMessage(LogLevel.INFO, "install BU包中无表格需要安装,退出,bu包:{0}",
					fileObject.getAbsolutePath());
			return;
		}
		String[] s = new String[sqls.size()];
		for (int i = 0; i < sqls.size(); i++) {
			s[i] = sqls.get(i);
		}
		JdbcTemplate jt = (JdbcTemplate) SpringUtil.getBean("jdbcTemplate");
		logger.logMessage(LogLevel.INFO, "开始执行sql,共{0}句sql", sqls.size());

		for (int i = 0; i < sqls.size(); i++) {
			logger.logMessage(LogLevel.INFO, "开始执行sql:{0}", sqls.get(i));

			try {
				jt.execute(sqls.get(i));
			} catch (Exception e) {
				// logger.logMessage(LogLevel.ERROR,
				// "sql执行出错:{0}",e.getMessage());
				logger.errorMessage("sql执行出错:{0}", e, sqls.get(i));
			}

		}

		logger.logMessage(LogLevel.INFO, "执行sql批已处理完成并提交");
		logger.logMessage(LogLevel.INFO, "install BU包中的表格完成,bu包:{0}",
				fileObject.getAbsolutePath());

	}

	private void deal(Table table, List<String> sqls, Connection connect) {
		if (tableList.contains(table))
			return;
		tableList.add(table);
		installTable(table, sqls, connect);
	}

	private void installTable(Table table, List<String> sqls, Connection connect) {
		logger.logMessage(LogLevel.INFO, "开始生成表格语句,表格 包:{0},名:{1}",
				table.getPackageName(), table.getName());
		String lanuage = getLanguage();
		try {
			DatabaseMetaData data = connect.getMetaData();
			// TableSqlProcessor sqlprocessor = new OracleSqlProcessorImpl();
			List<String> tableSqls = null;
			if (tableProcessor.checkTableExist(table, connect.getCatalog(),
					data, lanuage)) {
				tableSqls = tableProcessor.getUpdateSql(table,
						table.getPackageName(), data, connect.getCatalog(),
						lanuage);
			} else {
				tableSqls = tableProcessor.getCreateSql(table,
						table.getPackageName(), lanuage);
			}
			if (tableSqls.size() != 0) {
				logger.logMessage(LogLevel.INFO, "生成sql:{0}", tableSqls);
			} else {
				logger.logMessage(LogLevel.INFO, "无需生成Sql");
			}
			sqls.addAll(tableSqls);
			logger.logMessage(LogLevel.INFO, "生成表格语句完成,表格 包:{0},名:{1}",
					table.getPackageName(), table.getName());
		} catch (Exception e) {
			logger.errorMessage("获取表格sql时出错,表格 包:{0},名:{1}", e,
					table.getPackageName(), table.getName());
		}

	}

}
