/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.databasebuinstaller;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.tinygroup.database.config.view.View;
import org.tinygroup.database.util.DataBaseUtil;
import org.tinygroup.database.view.ViewProcessor;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.springutil.SpringUtil;
import org.tinygroup.vfs.FileObject;

public class ViewIntaller extends DbInstaller {
	private Logger loger = LoggerFactory.getLogger(TableInstaller.class);

	private ViewProcessor viewProcessor = SpringUtil
			.getBean(DataBaseUtil.VIEW_BEAN);

	public void process(FileObject fileObject) {
		loger.logMessage(LogLevel.INFO, "开始install BU包中的view,bu包:{0}",
				fileObject.getAbsolutePath());
		List<View> list = viewProcessor.getViews();
		List<String> sqls = new ArrayList<String>();
		SessionFactory sessionfactory = (SessionFactory) SpringUtil
				.getBean("sessionFactory");
		Session session = sessionfactory.openSession();
		Connection connect = session.connection();

		dealViews(list, sqls);
		try {
			Transaction tx = session.beginTransaction();
			Statement s = connect.createStatement();
			for (String sql : sqls) {
				s.execute(sql);
			}
			tx.commit();
			loger.logMessage(LogLevel.INFO, "install BU包中的视图完成,bu包:{0}",
					fileObject.getAbsolutePath());
		} catch (SQLException e) {
			loger.errorMessage("install BU包中的视图出错,bu包:{0}", e,
					fileObject.getAbsolutePath());
		}
	}

	private void dealViews(List<View> list, List<String> sqls) {
		for (View view : list) {
			sqls.add(viewProcessor.getCreateSql(view, getLanguage()));
		}
	}

}
